@@ -1,5 +1,6 @@
 from .imports import *
 from .path_utils import *
+from abstract_utilities import make_list
 _LOCK = threading.RLock()
 # ---------------- Registry ---------------- #
 
@@ -73,6 +74,7 @@ class infoRegistry(metaclass=SingletonMeta):
 
     def _write_cached_info(self, video_id: str, info: dict) -> str:
         cache_dir = self.video_root if self.flat_layout else os.path.join(self.video_root, video_id)
+        check_create_logs(f"making this directory == {cache_dir} == _write_cached_info line 77")
         os.makedirs(cache_dir, exist_ok=True)
         cache = os.path.join(cache_dir, "info.json")
         get_atomic_write(cache, info)
@@ -148,18 +150,55 @@ class infoRegistry(metaclass=SingletonMeta):
 
     def edit_info(self, data: dict, url: str | None = None,
                   video_id: str | None = None, video_path: str | None = None):
+        # Load current
         cur = self.get_video_info(url=url, video_id=video_id, video_path=video_path, force_refresh=False)
         if not cur:
             raise RuntimeError("No existing info to edit")
-        cur.update(data or {})
-        vid = cur.get("video_id") or video_id or (url and get_sha12(url)) or generate_video_id(video_path or "video")
-        cur = ensure_standard_paths(cur, self.video_root)
-        cache = self._write_cached_info(vid, cur)
-        self._link(vid, url, cur.get("file_path") or video_path)
-        cur["info_path"] = cache
-        cur["video_id"] = vid
-        return cur
 
+        # Merge edits
+        cur.update(data or {})
+
+        # --- Canonical, stable video_id (avoid lowercasing fallbacks) ---
+        canonical = (
+            cur.get("video_id")
+            or cur.get("id")
+            or video_id
+            or (get_sha12(url) if url else None)
+        )
+        if not canonical:
+            # last-resort only (try hard to avoid this to prevent new folders)
+            canonical = generate_video_id(video_path or "video")
+
+        cur["video_id"] = canonical
+
+        # --- Anchor schema to the REAL video folder (the one with video.mp4) ---
+        # Prefer explicit directory, else derive from file_path / video_path
+        video_dir = cur.get("directory")
+        if not video_dir:
+            fp = cur.get("file_path") or cur.get("video_path") or video_path
+            if fp:
+                video_dir = os.path.dirname(os.path.abspath(fp))
+
+        # If we know the real folder, expand schema IN PLACE (flat)
+        # This prevents creating a second <video_root>/<video_id> directory.
+        if video_dir:
+            cur = ensure_standard_paths(cur, video_dir, flat_layout=True)
+            cache_path = os.path.join(video_dir, "info.json")
+        else:
+            # Fallback: use registry root with configured layout
+            base = self.video_root if self.flat_layout else os.path.join(self.video_root, canonical)
+            os.makedirs(base, exist_ok=True)
+            cur = ensure_standard_paths(cur, base, flat_layout=self.flat_layout)
+            cache_path = os.path.join(base, "info.json")
+
+        # Write the single canonical info.json next to the video (or in fallback base)
+        get_atomic_write(cache_path, cur)
+
+        # Keep registry links in sync
+        self._link(canonical, url, cur.get("file_path") or video_path)
+
+        cur["info_path"] = cache_path
+        return cur
     def list_cached_videos(self):
         with _LOCK:
             return [
@@ -180,7 +219,7 @@ class VideoDownloader:
                  flat_layout: bool = False,video_url =None):
 
         self.video_url = self.url = get_corrected_url(url or video_url) 
-        self.video_urls = self.url if isinstance(self.video_url, list) else [self.url]
+        self.video_urls = make_list( self.video_url)
 
         self.registry = registry or infoRegistry(video_root=download_directory,flat_layout=flat_layout)
         self.ydl_opts = ydl_opts or {}
@@ -192,6 +231,9 @@ class VideoDownloader:
         self.force_refresh = force_refresh
         self.flat_layout = flat_layout   # 🔑
         self.video_path=video_path
+        self.info = self.registry.get_video_info(url=self.video_url, force_refresh=self.force_refresh)
+        self.video_path=self.info.get('video_path')
+        
         self.monitoring = True
         self.pause_event = threading.Event()
 
@@ -223,27 +265,29 @@ class VideoDownloader:
         logger.info("Monitor: exited.")
 
     def _build_ydl_opts(self, outtmpl, extractor_client=None):
-        fmt = "bestvideo+bestaudio/best"
-        if self.video_extention and self.video_extention != "mp4":
-            fmt = f"bestvideo[ext={self.video_extention}]+bestaudio[ext=m4a]/best[ext={self.video_extention}]"
-
         opts = {
             "quiet": True,
             "noprogress": True,
-            "external_downloader": "ffmpeg",
-            "outtmpl": outtmpl,
-            "format": fmt,
+            # write under <download_directory>/<id>/<id>.<ext>
+            "outtmpl": outtmpl,                      # see call-site below
+            # pick best and merge; final container = mp4
+            "format": "bestvideo+bestaudio/best",
             "merge_output_format": "mp4",
-            "concurrent_fragment_downloads": 3,
+            # remux to mp4 even when streams are webm/opus
+            "postprocessors": [
+                {"key": "FFmpegVideoRemuxer", "preferedformat": "mp4"},
+            ],
             "retries": 5,
             "fragment_retries": 5,
             "ignoreerrors": False,
+            # optional: YouTube client to reduce 403s
+            "extractor_args": {"youtube": {"player_client": ["android"]}},
         }
         if extractor_client:
             opts.setdefault("extractor_args", {}).setdefault("youtube", {})["player_client"] = [extractor_client]
         if self.user_agent:
             opts["http_headers"] = {"User-Agent": self.user_agent}
-        opts.update(self.ydl_opts)
+        opts.update(self.ydl_opts or {})
         return opts
 
     def _download_entrypoint(self):
@@ -258,85 +302,82 @@ class VideoDownloader:
                          video_path=None, output_filename=None, ydl_opts=None,
                          registry=None, force_refresh=False,
                          flat_layout: bool = False,video_url =None):
-         video_url = url = get_corrected_url(url or video_url or self.url or self.video_url) 
+        video_url = url = get_corrected_url(url or video_url or self.url or self.video_url) 
 
-         ydl_opts = ydl_opts or self.ydl_opts or {}
-         user_agent = user_agent or self.user_agent
-         
-         force_refresh = force_refresh or self.force_refresh
+        ydl_opts = ydl_opts or self.ydl_opts or {}
+        user_agent = user_agent or self.user_agent
+
+        force_refresh = force_refresh or self.force_refresh
          
          
-         download_directory = download_directory or self.download_directory
-         video_extention = video_extention or self.video_extention
-         output_filename = output_filename or self.output_filename
-         video_path = video_path or self.video_path
-         if video_path:
+        download_directory = download_directory or self.download_directory
+        video_extention = video_extention or self.video_extention
+        output_filename = output_filename or self.output_filename
+        video_path = video_path or self.video_path
+        if video_path:
              dirname = os.path.dirname(video_path)
              output_filename = dirname if dirname else download_directory or get_video_root(download_directory)
              basename = os.path.basename(video_path)
-             filename,ext = os.path.splitext(filename)
+             filename,ext = os.path.splitext(basename)
              output_filename = filename if filename else output_filename or 'video'
              video_extention = ext if ext else video_extention or '.mp4'
              
          
-         output_filename = output_filename or 'video'
-         download_directory = download_directory  or get_video_root(download_directory)
-         flat_layout = flat_layout or self.flat_layout
-         registry = registry or self.registry or infoRegistry(
+        output_filename = output_filename or 'video'
+        download_directory = download_directory  or get_video_root(download_directory)
+        flat_layout = flat_layout or self.flat_layout
+        registry = registry or self.registry or infoRegistry(
              video_root=download_directory,
              flat_layout=flat_layout
              )
-         logger.info(f"[VideoDownloader] Processing: {video_url}")
-         video_url = get_corrected_url(url or video_url or self.url or self.video_url) 
-         if "youtube.com/watch" in video_url and "v=" not in video_url:
+        logger.info(f"[VideoDownloader] Processing: {video_url}")
+        video_url = get_corrected_url(url or video_url or self.url or self.video_url) 
+        if "youtube.com/watch" in video_url and "v=" not in video_url:
             logger.debug(f"[VideoDownloader] Skipping bare watch URL: {video_url}")
             return None
 
-         info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
-         if info and info.get("video_path") and os.path.isfile(info["video_path"]):
-            logger.info(f"[VideoDownloader] Already cached: {info['video_path']}")
-            return info
-
-         try:
-            video_extention = video_extention or "mp4"
-            outtmpl = os.path.join(self.download_directory, "%(id)s.%(video_extention)s")
-            with yt_dlp.YoutubeDL(self._build_ydl_opts(outtmpl)) as ydl:
-                raw_info = ydl.extract_info(video_url, download=self.get_download)
-
-            video_id = raw_info.get("id") or generate_video_id(raw_info.get("title") or "video")
-
-            temp_path = ydl.prepare_filename(raw_info)
-            _, ext = os.path.splitext(temp_path)
-            ext = (ext.lstrip(".") or raw_info.get("ext") or video_extention).lower()
-            if not re.match(r"^[a-z0-9]+$", ext):
-                logger.warning(f"[VideoDownloader] Invalid ext {ext}, forcing mp4")
-                ext = video_extention or "mp4"
-            directory =info.get('directory')
-            final_path = os.path.join(directory, f"{output_filename}.{ext}")
-            if temp_path != final_path and os.path.isfile(temp_path):
-                shutil.move(temp_path, final_path)
-
-            minimal_info = {
-                "id": raw_info.get("id"),
-                "title": raw_info.get("title"),
-                "ext": ext,
-                "duration": raw_info.get("duration"),
-                "upload_date": raw_info.get("upload_date"),
-                "video_id": video_id,
-                "video_path": final_path,
-                "file_path": final_path,
-            }
-            self.registry.edit_info(minimal_info, url=video_url,
-                                    video_id=video_id, video_path=final_path)
-
-            info = self.registry.get_video_info(video_id=video_id)
-            logger.info(f"[VideoDownloader] Stored in registry at {info['video_path']}")
-            return info
-
-
-         except Exception as e:
-            logger.error(f"[VideoDownloader] Download failed: {e}")
+        info = self.registry.get_video_info(url=video_url, force_refresh=self.force_refresh)
+        if not info:
+            logger.error("[VideoDownloader] No info; cannot determine target directory")
             return None
+
+        directory = info.get("directory") or os.path.join(self.download_directory, info.get("id", "video"))
+        check_create_logs(f"making this directory == {directory} line 316")
+        os.makedirs(directory, exist_ok=True)
+
+        # force an actual yt-dlp field: %(ext)s (not %(video_extention)s)
+        # this will become video.mp4 because merge_output_format='mp4'
+        outtmpl = os.path.join(directory, "video.%(ext)s")
+
+        with yt_dlp.YoutubeDL(self._build_ydl_opts(outtmpl)) as ydl:
+            raw_info = ydl.extract_info(video_url, download=self.get_download)
+
+        # compute final path (should already be video.mp4 with the template above)
+        temp_path = ydl.prepare_filename(raw_info)
+        final_path = os.path.join(directory, "video.mp4")
+        if os.path.abspath(temp_path) != os.path.abspath(final_path) and os.path.isfile(temp_path):
+            shutil.move(temp_path, final_path)
+
+        # minimal info for registry
+        video_id = raw_info.get("id") or generate_video_id(raw_info.get("title") or "video")
+        minimal_info = {
+            "id": raw_info.get("id"),
+            "title": raw_info.get("title"),
+            "ext": "mp4",
+            "duration": raw_info.get("duration"),
+            "upload_date": raw_info.get("upload_date"),
+            "video_id": video_id,
+            "video_path": final_path,
+            "file_path": final_path,
+        }
+        self.registry.edit_info(minimal_info, url=video_url, video_id=video_id, video_path=final_path)
+
+
+        info = self.registry.get_video_info(video_id=video_id)
+        logger.info(f"[VideoDownloader] Stored in registry at {info['video_path']}")
+        return info
+
+
 def get_videoDownloader(url=None,
                    download_directory=None,
                    user_agent=None,
@@ -508,7 +549,7 @@ def get_video_filepath(
     if download:
         VideoDownloader(url=url)
     return get_video_info_spec(
-        key='filepath',
+        key='video_path',
         url=url,
         video_id=video_id,
         force_refresh=force_refresh,
