@@ -25,6 +25,23 @@ VIDEO_SCHEMA = {
         "frames": "{video_id}_frame_{i}.jpg",  # pattern
     }
 }
+def check_create_logs(string):
+    stri = string.split('==')[1]
+    directory = eatAll(stri,[' ','\t'])
+    if directory.lower() == directory:
+        logger.info(f"this is the directory fudd\n\n\n{string}")
+    
+def generate_video_id(path_or_title: str, max_length: int = 50) -> str:
+    base = os.path.splitext(os.path.basename(path_or_title))[0]
+    if base == 'video':
+        base = os.path.basename(os.path.dirname(path_or_title)) or base
+    base = get_normalize_ascii(base.lower())
+    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
+    base = re.sub(r'-{2,}', '-', base)
+    if len(base) > max_length:
+        h = get_sha12(base)
+        base = f"{base[:max_length - len(h) - 1].rstrip('-')}-{h}"
+    return base or get_sha12(path_or_title)
 def get_temp_id(url):
     url = str(url)
     url_length = len(url)
@@ -183,70 +200,120 @@ def for_dl_video(url,download_directory=None,output_filename=None,get_info=None,
         video_mgr = dl_video(src,download_directory=download_directory,output_filename=output_filename,download_video=download_video)
         if get_video_info_from_mgr(video_mgr):
             return video_mgr
-def downloadvideo(url,
-                  directory=None,
-                  output_filename=None,
-                  rename_display=None,
-                  thumbnails=None,
-                  audio=None,
-                  safari_optimize=None,
-                  download_video=None,
-                  flat_layout: bool = False,
-                  *args, **kwargs):
+def downloadvideo(
+    url,
+    directory=None,
+    output_filename=None,          # kept for API compat; not used for final name
+    rename_display=None,           # kept for API compat; final file is always video.mp4
+    thumbnails=None,
+    audio=None,
+    safari_optimize=None,
+    download_video=None,
+    flat_layout: bool = False,
+    *args, **kwargs,
+):
     """
     Download a video and save alongside info.json and other schema files.
-    If flat_layout=True, everything goes into `directory` directly.
+
+    Rules:
+      - Final file is ALWAYS <video_dir>/video.mp4
+      - Only ONE canonical video_dir is used
+      - No extra folders derived from 'display_id' are created
     """
-    rename_display = bool_or_default(rename_display)
-    thumbnails = bool_or_default(thumbnails)
-    audio = bool_or_default(audio, default=False)
-    safari_optimize = bool_or_default(safari_optimize, default=True)
-    download_video = bool_or_default(download_video, default=True)
+    # normalize flags
+    rename_display   = bool_or_default(rename_display)          # no-op for file name
+    thumbnails       = bool_or_default(thumbnails)
+    audio            = bool_or_default(audio, default=False)
+    safari_optimize  = bool_or_default(safari_optimize, default=True)
+    download_video   = bool_or_default(download_video, default=True)
 
-    output_filename = output_filename or get_temp_file_name(url)
-    video_mgr = for_dl_video(url,
-                             download_directory=directory,
-                             output_filename=output_filename,
-                             download_video=download_video)
-    info = video_mgr.info
-    display_id = get_display_id(info)
+    # Kick off fetch/download via your existing flow
+    # (keeps your yt-dlp + registry logic intact)
+    _tmp_name = output_filename or get_temp_file_name(url)
+    video_mgr = for_dl_video(
+        url,
+        download_directory=directory,
+        output_filename=_tmp_name,
+        download_video=download_video,
+    )
+    info = get_video_info_from_mgr(video_mgr) or {}
 
-    os.makedirs(directory, exist_ok=True)
-    video_directory = directory if flat_layout else os.path.join(directory, display_id)
-    os.makedirs(video_directory, exist_ok=True)
-    info['file_path'] = video_directory
-    if info:
-        file_path = info.get('file_path')
-    if rename_display and file_path:
-        # Rename using metadata
-        video_id = info.get('id', get_temp_id(url))
-        title = output_filename or get_video_title(info)
-        safe_title = get_safe_title(title)
-        final_filename = output_filename or f"{safe_title}_{video_id}"
-        final_filename = f"{final_filename}.mp4"
-        new_path = os.path.join(video_directory, final_filename)
-        if os.path.exists(info['file_path']):
-            os.rename(info['file_path'], new_path)
-            info['file_path'] = new_path
-        info['file_path'] = new_path
-            
-            # *** Here we call the optimization function ***
-    video_path = info.get('file_path')
-    if video_path and video_path.lower().endswith('.mp4') and safari_optimize:
-        info['file_path'] = optimize_video_for_safari(video_path,reencode=safari_optimize)
-    info_path = os.path.join(video_directory, 'info.json')
+    # Decide canonical id & base root
+    video_id   = info.get("video_id") or info.get("id") or get_temp_id(url)
+    # If caller passed a root directory, respect it; otherwise use what the registry established
+    base_root  = directory or info.get("directory") or get_video_root(None)
+
+    # Decide the single canonical directory to use
+    video_dir  = base_root if flat_layout else os.path.join(base_root, video_id)
+    logger.info(f"making this downloadvideo == {video_dir} == get_video_directory line 246")
+    os.makedirs(video_dir, exist_ok=True)
+
+    # Where the final file must live
+    final_path = os.path.join(video_dir, "video.mp4")
+
+    # If a file was downloaded somewhere else, move it into place
+    cur_path = info.get("file_path") or info.get("video_path")
+    if cur_path and os.path.isfile(cur_path):
+        if os.path.abspath(cur_path) != os.path.abspath(final_path):
+            final_dir = os.path.dirname(final_path)
+            logger.info(f"making this downloadvideo == {final_dir} == get_video_directory line 251")
+            os.makedirs(final_dir, exist_ok=True)
+            try:
+                shutil.move(cur_path, final_path)
+            except Exception:
+                # If move fails (e.g. cross-device), copy+remove
+                shutil.copy2(cur_path, final_path)
+                os.remove(cur_path)
+    else:
+        # If yt-dlp already wrote video.mp4 at the canonical spot, keep it.
+        # Otherwise, try to find any video-like file in video_dir and rename it.
+        if not os.path.isfile(final_path):
+            candidates = []
+            for name in os.listdir(video_dir):
+                p = os.path.join(video_dir, name)
+                if os.path.isfile(p) and os.path.splitext(name)[1].lower() in {".mp4", ".m4v", ".mov", ".webm", ".mkv"}:
+                    candidates.append(p)
+            if candidates:
+                # choose the largest as the likely full file
+                best = max(candidates, key=lambda p: os.path.getsize(p))
+                if os.path.abspath(best) != os.path.abspath(final_path):
+                    shutil.move(best, final_path)
+
+    # Safari faststart (moov up front) if we have a proper mp4
+    if os.path.isfile(final_path) and final_path.lower().endswith(".mp4") and safari_optimize:
+        try:
+            optimize_video_for_safari(final_path, reencode=safari_optimize)
+        except Exception as e:
+            print(f"Safari optimization skipped: {e}")
+
+    # Update info with canonical paths
+    info["video_id"]  = video_id
+    info["directory"] = video_dir
+    info["file_path"] = final_path
+    info["video_path"]= final_path
+
+    # Thumbnails / audio (optional)
     if thumbnails:
-        info = get_thumbnails(video_directory, info)
+        try:
+            info = get_thumbnails(video_dir, info)
+        except Exception as e:
+            print(f"Thumbnail fetch failed: {e}")
     if audio:
         try:
-            info = download_audio(directory, info)
-        except:
-            info['audio_path'] = None
-    info['json_path'] = os.path.join(video_directory, 'info.json')
-    safe_dump_to_file(info, info['json_path'])
+            info = download_audio(video_dir, info)  # your helper
+        except Exception:
+            info["audio_path"] = None
+
+    # Write info.json next to the video
+    info_path = os.path.join(video_dir, "info.json")
+    info["info_path"] = info_path
+    info["json_path"] = info_path
+    safe_dump_to_file(info, info_path)
+
+    # Attach the rest of the schema INSIDE the already-chosen video_dir
+    info = ensure_standard_paths(info, video_dir, flat_layout=True)
+    return info
 
-    # attach schema paths with flat awareness
-    info = ensure_standard_paths(info, directory, flat_layout=flat_layout)
     return info
 
 
@@ -262,6 +329,7 @@ def expand_schema(video_id: str, folder: str, schema: dict[str, Any], flat_layou
             dirname = key.replace("_dir", "").replace("_directory", "")
             # If flat, stay in current folder
             subfolder = folder if flat_layout else os.path.join(folder, dirname)
+            check_create_logs(f"making this directory == {subfolder} == expand_schema line 323")
             os.makedirs(subfolder, exist_ok=True)
             result[f"{dirname}_directory"] = subfolder
             result[key] = expand_schema(video_id, subfolder, rel, flat_layout=flat_layout)
@@ -284,6 +352,7 @@ def ensure_standard_paths(info: dict, video_root: str, flat_layout: bool = False
         return info
 
     dirbase = video_root if flat_layout else os.path.join(video_root, vid)
+    check_create_logs(f"making this directory == {dirbase} == ensure_standard_paths line 347")
     os.makedirs(dirbase, exist_ok=True)
     info["directory"] = dirbase
 
@@ -313,6 +382,7 @@ def get_video_folder(video_id, envPath=None, flat_layout: bool = False):
     """Return the canonical video folder (with optional flat layout)."""
     root = get_video_directory(envPath=envPath)
     dir_path = root if flat_layout else os.path.join(root, video_id)
+    check_create_logs(f"making this directory == {dir_path} == ensure_standard_paths line 377")
     os.makedirs(dir_path, exist_ok=True)
     return dir_path
 
@@ -348,7 +418,7 @@ def get_video_directory(key=None, envPath=None):
     video_directory = get_video_env(key=key, envPath=envPath)
     if not video_directory:
         video_directory = get_video_root()
-
+    check_create_logs(f"making this directory == {dir_path} == get_video_directory line 413")
     os.makedirs(video_directory, exist_ok=True)
     return video_directory
 
@@ -371,25 +441,25 @@ def get_video_paths(video_id, envPath=None):
     return {key: os.path.join(folder, rel) for key, rel in VIDEO_SCHEMA.items()}
 
 
-def generate_video_id(path: str, max_length: int = 50) -> str:
-    # 1. Take basename (no extension)
-    file_parts = get_file_parts(path)
-    base= file_parts.get("filename")
-    if base == 'video':
-        base = file_parts.get("dirbase")
-    # 2. Normalize Unicode → ASCII
-    base = unicodedata.normalize('NFKD', base).encode('ascii', 'ignore').decode('ascii')
-    # 3. Lower-case
-    base = base.lower()
-    # 4. Replace non-alphanumeric with hyphens
-    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
-    # 5. Collapse duplicates
-    base = re.sub(r'-{2,}', '-', base)
-    # 6. Optionally truncate & hash for uniqueness
-    if len(base) > max_length:
-        h = hashlib.sha1(base.encode()).hexdigest()[:8]
-        base = base[: max_length - len(h) - 1].rstrip('-') + '-' + h
-    return base
+##def generate_video_id(path: str, max_length: int = 50) -> str:
+##    # 1. Take basename (no extension)
+##    file_parts = get_file_parts(path)
+##    base= file_parts.get("filename")
+##    if base == 'video':
+##        base = file_parts.get("dirbase")
+##    # 2. Normalize Unicode → ASCII
+##    base = unicodedata.normalize('NFKD', base).encode('ascii', 'ignore').decode('ascii')
+##    # 3. Lower-case
+##    base = base.lower()
+##    # 4. Replace non-alphanumeric with hyphens
+##    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
+##    # 5. Collapse duplicates
+##    base = re.sub(r'-{2,}', '-', base)
+##    # 6. Optionally truncate & hash for uniqueness
+##    if len(base) > max_length:
+##        h = hashlib.sha1(base.encode()).hexdigest()[:8]
+##        base = base[: max_length - len(h) - 1].rstrip('-') + '-' + h
+##    return base
 def _get_video_info(url=None,file_path=None, ydl_opts=None,output_filename=None, cookies_path=None):
     from yt_dlp import YoutubeDL
     
@@ -421,17 +491,7 @@ def get_atomic_write(path: str, data: dict):
 def get_normalize_ascii(s: str) -> str:
     return unicodedata.normalize('NFKD', s).encode('ascii', 'ignore').decode('ascii')
 
-def generate_video_id(path_or_title: str, max_length: int = 50) -> str:
-    base = os.path.splitext(os.path.basename(path_or_title))[0]
-    if base == 'video':
-        base = os.path.basename(os.path.dirname(path_or_title)) or base
-    base = get_normalize_ascii(base.lower())
-    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
-    base = re.sub(r'-{2,}', '-', base)
-    if len(base) > max_length:
-        h = get_sha12(base)
-        base = f"{base[:max_length - len(h) - 1].rstrip('-')}-{h}"
-    return base or get_sha12(path_or_title)
+
 
 def make_video_info(filepath: str) -> dict:
     import json as _json
